package eu.eudat.logic.mapper.elastic;

import eu.eudat.data.dao.criteria.DataManagementPlanCriteria;
import eu.eudat.data.entities.DMP;
import eu.eudat.elastic.criteria.DatasetCriteria;
import eu.eudat.elastic.entities.Collaborator;
import eu.eudat.elastic.entities.Dataset;
import eu.eudat.elastic.entities.Organization;
import eu.eudat.elastic.entities.Tag;
import eu.eudat.logic.managers.DatasetManager;
import eu.eudat.logic.services.ApiContext;
import eu.eudat.logic.utilities.helpers.StreamDistinctBy;

import java.util.*;
import java.util.stream.Collectors;

public class DatasetMapper {

	private final ApiContext apiContext;
	private final DatasetManager datasetManager;

	public DatasetMapper(ApiContext apiContext, DatasetManager datasetManager) {
		this.apiContext = apiContext;
		this.datasetManager = datasetManager;
	}

	public Dataset toElastic(eu.eudat.data.entities.Dataset dataset, List<Tag> tags) throws Exception {
		Dataset elastic = new Dataset();
		eu.eudat.data.entities.Dataset tempDataset = apiContext.getOperationsContext().getDatabaseRepository().getDatasetDao().find(dataset.getId());
		elastic.setId(tempDataset.getId().toString());
		if (tags != null && !tags.isEmpty()) {
			DatasetCriteria criteria = new DatasetCriteria();
			criteria.setTags(tags);
			List<Tag> tags1 = apiContext.getOperationsContext().getElasticRepository().getDatasetRepository().query(criteria).stream().map(eu.eudat.elastic.entities.Dataset::getTags).flatMap(Collection::stream)
									   .filter(StreamDistinctBy.distinctByKey(Tag::getId)).filter(tag -> tags.stream().anyMatch(tag1 -> tag1.getName().equals(tag.getName()))).collect(Collectors.toList());
			if (tags1.isEmpty()) {
				tags.forEach(tag -> tag.setId(UUID.randomUUID().toString()));
				elastic.setTags(tags);
			} else {
				elastic.setTags(tags1);
			}
		}
		elastic.setLabel(tempDataset.getLabel());
		elastic.setDescription(tempDataset.getDescription());
		elastic.setTemplate(tempDataset.getProfile().getId());
		elastic.setStatus(tempDataset.getStatus());
		elastic.setDmp(tempDataset.getDmp().getId());
		elastic.setGroup(tempDataset.getDmp().getGroupId());
		elastic.setGrant(tempDataset.getDmp().getGrant().getId());
		if (tempDataset.getDmp().getUsers() != null) {
			elastic.setCollaborators(tempDataset.getDmp().getUsers().stream().map(user -> CollaboratorMapper.toElastic(user.getUser())).collect(Collectors.toList()));
		}
		DataManagementPlanCriteria dmpCriteria = new DataManagementPlanCriteria();
		dmpCriteria.setAllVersions(true);
		dmpCriteria.setGroupIds(Collections.singletonList(tempDataset.getDmp().getGroupId()));
		apiContext.getOperationsContext().getDatabaseRepository().getDmpDao().getWithCriteria(dmpCriteria).toList().stream()
				  .max(Comparator.comparing(DMP::getVersion)).ifPresent(dmp -> elastic.setLastVersion(dmp.getId().equals(tempDataset.getDmp().getId())));
		apiContext.getOperationsContext().getDatabaseRepository().getDmpDao().getWithCriteria(dmpCriteria).toList().stream().filter(DMP::isPublic)
				  .max(Comparator.comparing(DMP::getVersion)).ifPresent(dmp -> elastic.setLastPublicVersion(dmp.getId().equals(tempDataset.getDmp().getId())));
		if (elastic.getLastVersion() == null) {
			elastic.setLastVersion(true);
		}
		if (elastic.getLastPublicVersion() == null) {
			elastic.setLastPublicVersion(false);
		}
		if (tempDataset.getDmp().getOrganisations() != null) {
			elastic.setOrganizations(tempDataset.getDmp().getOrganisations().stream().map(OrganizationMapper::toElastic).collect(Collectors.toList()));
		}
		elastic.setPublic(tempDataset.getDmp().isPublic());
		elastic.setGrantStatus(tempDataset.getDmp().getGrant().getStatus());
		elastic.setFormData(datasetManager.getWordDocumentText(tempDataset));

		return elastic;
	}
}
